 DEFINE PAGE00, SPACE=ROM
 SEGMENT PAGE00

 public Reboot, Boot
 extern PutChar, DrawString, KeyboardInit, MeasureString
 extern LCDInit, ClearBuffer, LCDDelay, FastCopy
 extern WaitForever, GetKey, WaitKey, FlushKeys
 extern OSVersion, TitleScreen, FSInit, OSName
 extern ResetTextLocation, Sleep, Random

 nolist
 include "WFRNGOS.inc"
 include "Keys.inc"
InputBuffer	equ	9001h ; IT'S OVER 9000!!!!1!!!!11!
 list

	jr Boot
  	db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
 	db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0			; 55 zeroes
	jr SysInterrupt
 	db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	db 0,0,0,0,0,0								; 27 zeroes
	jp Boot
	db 0FFh, 0FFh, 5Ah, 0A5h
Boot:	; 0058h
	; This is called as soon as the batteries hit the calc,
	; not when ON is pressed
	call FSInit
	; Sleep requires memory to be properly arranged (hence FSInit)
	; and requires the stack to be initialized
	ld sp, 0FFFFh		; Initialize the stack
	call ClearBuffer
	call Sleep
Reboot:
	ld iy, FlagStart	; Initialize flags
	
	;Get the calculator model
	in a, (2)
 	rlca ;Roll bit 7 into carry.
	jr nc, Regular83Plus
	and 40h ;Test bit 5, now rotated into bit 6.
	jr z, Silver83Plus
	in a, (21h)
	and 03h
	jr z, Regular84Plus
	; 84+ SE
	ld a, 3
	jr CheckDone
	Silver83Plus:
	ld a, 1
	jr CheckDone
	Regular84Plus:
	ld a, 2
	jr CheckDone
	Regular83Plus:
	ld a, 0
	CheckDone:
	ld (ModelID), a

	; Load drivers
	call LCDInit
	call KeyboardInit

	; This is where the OS start-up code goes.
	; Delete it and replace it with your own boot code.
	call ClearBuffer
	call ResetTextLocation
	ld hl, OSName
	call DrawString
	call FastCopy

	call Reset
	push af
GameLoop:
	pop af
	call FlushKeys
	call WaitKey
	cp kClear
	push af
	jr z, Boot ; Clear to turn off the calc
	pop af
	cp kEnter
	jr z, SubmitValue
	sub a, 12h ; Move the key value down to the first numeric key
	cp 13h
	push af
	jr nc, GameLoop ; Non-numeric keys are unsupported
	pop af
	ld hl, KeyTable ; Get the key table in HL

	add a,l
	ld l,a
 	jr nc,$+3
	inc h    ; And add the key value to HL

	ld a, (hl)
	cp '.'
	jr z, GameLoop ; . is an non-numeric key

	push af
	ld a, c
	cp 2 ; 2 digits max
	jr z, GameLoop
	pop af
	inc c
	ld (de), a
	inc de

	call PutChar ; Draw the character on-screen
	call FastCopy
	push af
	jr GameLoop
SubmitValue:
	ld de, InputBuffer
	ld a, c
	or a
	push af
	jr z, GameLoop ; Must have at least one character entered
	pop af
	cp 1
	jr nz, $+3
	dec de
;	di
;	halt
	call ASCIIToDec2
	ld a, l
	cp b
	jr z, Correct
	ld hl, IncorrectMessage
	call DrawString
	call FastCopy
	push af
	ld c, 0
	jr GameLoop
Correct:
	ld hl, CorrectMessage
	call DrawString
	call FastCopy
	push af
	;call Reset
	call FlushKeys
	call WaitKey
	cp kClear
	jr z, Boot
	call FlushKeys
	jr Reboot

Reset:
	ld de, InputBuffer ; DE is the input buffer
	ld hl, InputBuffer - 1
	ld (hl), '0'
	ld bc, 4
	ldir			; Clear the input buffer
	ld de, InputBuffer
	; Main Loop
	call Random
	ld hl, RandomNumberTable
	add a,l
	ld l,a
 	jr nc,$+3
	inc h    ; ADD HL, A
	ld a, (HL)
	ld b, a ; B is number to guess
	ld c, 0 ; C is the length of the current input string
	ret

CorrectMessage:
	db "\nYES!\nSIr cmPWN's tIcaLc.OrG\npassword is *******\npREss {clEAr} 2 shut\ndown\npReSS aNythING 3ls3\n2 PLay agAIn", 0
IncorrectMessage:
	db "\nNO, tHat's nOt It!\n", 0

; For speed's sake, we use a table for calculating random numbers between 1 and 30
RandomNumberTable:
	db 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15
	db 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30
	db 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15
	db 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30
	db 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15
	db 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30
	db 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15
	db 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30
	db 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15
	db 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30
	db 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15
	db 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30
	db 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15
	db 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30
	db 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15
	db 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30
	db 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15
	db 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30
; DE == pointer to number to convert (up to two digits)
; HL == Output
ASCIIToDec2:
	push de
	push bc
	push af
	call ASCIIToDec2Routine
	pop af
	pop bc
	pop de
	ret
ASCIIToDec2Routine:
	ld hl, 0
	ld bc, 10
	call NumConvert2
	ld bc, 1
NumConvert2:
	ld a, (de)
	sub a, '0'
ConvertLoop2:
	or a
	jr z, ConvertLoopDone2
	add hl, bc
	dec a
	jr ConvertLoop2
ConvertLoopDone2:
	inc de
	ret

KeyTable:
	db "369.....258....0147"

SignedDecText:
	call AbsHL
	call nz, AddNeg
	call DecText
	ex hl, de
	call DrawString
	ret
AddNeg:
	ld a, '-'
	call PutChar
	ret

AbsHL:
	push af
	bit 7,h
	jr z,positive
	xor a
	sub l
	ld l,a
	sbc a,a
	sub h
	ld h,a
	ld a, 1
	cp 0
positive:
	pop af
	ret

; DE == pointer to number to convert
; HL == Output
ASCIIToDec:
	ld hl, 0
	ld bc, 10000
	call NumConvert
	ld bc, 1000
	call NumConvert
	ld bc, 100
	call NumConvert
	ld bc, 10
	call NumConvert
	ld bc, 1
NumConvert:
	ld a, (de)
	sub a, '0'
ConvertLoop:
	or a
	jr z, ConvertLoopDone
	add hl, bc
	dec a
	jr ConvertLoop
ConvertLoopDone:
	inc de
	ret

DecText:
	push hl
	push de
	push af
	push bc
	ld de, NumberBuffer
	call Num2Dec
	ld de, NumberBuffer
DecLoop:
	ld a, (de)
	cp '0'
	ret nz
	cp 0
	jr z, DecLoopBackUp
	inc de
	jr DecLoop
DecLoopBackUp:
	dec de
	pop bc
	pop af
	pop de
	pop hl
	ret

Num2Dec:
	ld	bc,-10000
	call	Num1
	ld	bc,-1000
	call	Num1
	ld	bc,-100
	call	Num1
	ld	c,-10
	call	Num1
	ld	c,b

Num1:
	ld	a,'0'-1
Num2:
	inc	a
	add	hl,bc
	jr	c,Num2
	sbc	hl,bc

	ld	(de),a
	inc de
	ret

; System Interrupt Routines
SysInterrupt:
	exx
	ex af, af'
	in a, (04h)
	bit 0, a
	jr nz, HandleON
	bit 1, a
	jr nz, HandleTimer1
	bit 2, a
	jr nz, HandleTimer2
	bit 4, a
	jr nz, HandleLink
	jr InterruptDone
HandleON:
	in a, (03h)
	res 0, a
	out (03h), a
	set 0, a
	out (03h), a
	; ON interrupt
	jr InterruptDone
HandleTimer1:
	in a, (03h)
	res 1, a
	out (03h), a
	set 1, a
	out (03h), a
	; Timer one interrupt (might be best to merge with timer 2)
	jr InterruptDone
HandleTimer2:
	in a, (03h)
	res 2, a
	out (03h), a
	set 2, a
	out (03h), a
	; Timer two interrupt
	jr InterruptDone
HandleLink:
	in a, (03h)
	res 4, a
	out (03h), a
	set 4, a
	out (03h), a
	; Link interrupt
InterruptDone:
	ex af, af'
	exx
	ei
	ret
