 DEFINE PAGE00, SPACE=ROM
 SEGMENT PAGE00

 ; Knight OS
 ; Low-level LCD driver
 ; 5/24/2010

 include "WFRNGOS.inc"
 public LCDInit, ClearBuffer, LCDDelay, FastCopy, PutSpriteOR, PutSpriteXOR, LCDTest, LCDFix
 extern WaitKey, FlushKeys
 
; Initializes the LCD
LCDInit:
	ld a, 40h
	out (10h), a
	ld a, 05h
	call LCDDelay
	out (10h), a
	ld a, 01h
	call LCDDelay
	out (10h), a
	ld a, 3
	call LCDDelay
	out (10h), a
	ld a, 0F6h
	call LCDDelay
	out (10h), a
	set LCDDriver, (IY + DriverFlags)
	ret

LCDTest:
	ld a, 1Ch
	out (10h), a
	call FlushKeys
	call WaitKey
	call FlushKeys
	ld a, 18h
	out (10h), a
	ret

LCDFix:
	xor	a
	out	(20h), a
	ld	c, 0Ch-04h
delayTest:
	ld	d, 100
	ld	b, 255
	ld	a, 4
	add	a, c
	ld	c, a
	out	(29h), a
testLoop:
	inc	hl
	dec	hl
	inc	hl
	dec	hl
	inc	hl
	dec	hl	
	in	a, (10h)	;60 tstates
	or	a
	jp	m, delayTest
	ld	a, 01h
	out	(10h), a
	djnz	testLoop
	ld	b, 255
	dec	d
	jr	nz, testLoop	;25500 trials, 
	ret

ClearBuffer:
	ld hl, LCDBuffer
	ld bc, 768
clearBufferLoop:
	ld (hl), 0
	ld a, b
	dec bc
	inc hl
	cp 0
	jr nz, clearBufferLoop
	ld a, c
	cp 0
	jr nz, clearBufferLoop
	ret
	
; Do nothing for a few cycles
LCDDelay:
	push hl
	push de
	pop de
	pop hl
	nop
	ret

FastCopy:
	push hl
	push de
	push bc
	push af
	ld hl,LCDBuffer  ;This can be commented out or another entry placed after it
                           ;so the buffer can be provided by option.
	ld c,010h
	ld a,080h
setrow:
	in f,(c)
	jp m,setrow
	out (10h),a
	ld de,12
	ld a,20h
col:
	in f,(c)
	jp m,col
	out (10h),a
	push af
	ld b,64
row:
	ld a,(hl)
rowwait:
	in f,(c)
	jp m,rowwait
	out (11h),a
	add hl,de
	djnz row
	pop af
	dec h
	dec h
	dec h
	inc hl
	inc a
	cp 2ch
	jp nz,col
	pop af
	pop bc
	pop de
	pop hl
	ret

PutSpriteXOR:
; D = xpos
; E = ypos
; B = height
; IX = image address
; Start by doing vertical clipping
    LD     A, 11111111b         ; Reset clipping mask
    LD     (clip_mask), A
    LD     A, E                 ; If ypos is negative
    OR     A                    ; try clipping the top
    JP     M, ClipTop           ;
 
    SUB    64                   ; If ypos is >= 64
    RET    NC                   ; sprite is off-screen

    NEG                         ; If (64 - ypos) > height
    CP     B                    ; don't need to clip
    JR     NC, VertClipDone     ; 

    LD     B, A                 ; Do bottom clipping by
    JR     VertClipDone         ; setting height to (64 - ypos)

ClipTop:
    LD     A, B                 ; If ypos <= -height
    NEG                         ; sprite is off-screen
    SUB    E                    ;
    RET    NC                   ;

    PUSH   AF
    ADD    A, B                 ; Get the number of clipped rows
    LD     E, 0                 ; Set ypos to 0 (top of screen)
    LD     B, E                 ; Advance image data pointer
    LD     C, A                 ;
    ADD    IX, BC               ;
    POP    AF
    NEG                         ; Get the number of visible rows
    LD     B, A                 ; and set as height

VertClipDone:
; Now we're doing horizontal clipping
    LD     C, 0                 ; Reset correction factor
    LD     A, D

    CP     -7                   ; If 0 > xpos >= -7
    JR     NC, ClipLeft         ; clip the left side

    CP     96                   ; If xpos >= 96
    RET    NC                   ; sprite is off-screen

    CP     89                   ; If 0 <= xpos < 89
    JR     C, HorizClipDone     ; don't need to clip

ClipRight:
    AND    7                    ; Determine the clipping mask
    LD     C, A
    LD     A, 11111111b
FindRightMask:
    ADD    A, A
    DEC    C
    JR     NZ, FindRightMask
    LD     (clip_mask), A
    LD     A, D
    JR     HorizClipDone

ClipLeft:
    AND    7                    ; Determine the clipping mask
    LD     C, A
    LD     A, 11111111b
FindLeftMask:
    ADD    A, A
    DEC    C
    JR     NZ, FindLeftMask
    CPL
    LD     (clip_mask), A
    LD     A, D
    ADD    A, 96                ; Set xpos so sprite will "spill over"
    LD     C, 12                ; Set correction

HorizClipDone:
; A = xpos
; E = ypos
; B = height
; IX = image address

; Now we can finally display the sprite.
    LD     H, 0
    LD     D, H
    LD     L, E
    ADD    HL, HL
    ADD    HL, DE
    ADD    HL, HL
    ADD    HL, HL

    LD     E, A
    SRL    E
    SRL    E
    SRL    E
    ADD    HL, DE

    LD     DE, LCDBuffer
    ADD    HL, DE

    LD     D, 0                 ; Correct graph buffer address
    LD     E, C                 ; if clipping the left side
    SBC    HL, DE               ;

    AND    7
    JR     Z, _Aligned

    LD     C, A
    LD     DE, 11

_RowLoop:
    PUSH   BC
    LD     B, C
    LD     A, (clip_mask)       ; Mask out the part of the sprite
    AND    (IX)                 ; to be horizontally clipped
    LD     C, 0

_ShiftLoop:
    SRL    A
    RR     C
    DJNZ   _ShiftLoop

    XOR    (HL)
    LD     (HL), A

    INC    HL
    LD     A, C
    XOR    (HL)
    LD     (HL), A

    ADD    HL, DE
    INC    IX
    POP    BC
    DJNZ   _RowLoop
    RET

_Aligned:
    LD     DE, 12

_PutLoop:
    LD     A, (IX)
    XOR    (HL)
    LD     (HL), A
    INC    IX
    ADD    HL, DE
    DJNZ   _PutLoop
    RET
	
PutSpriteOR:
; D = xpos
; E = ypos
; B = height
; IX = image address
; Start by doing vertical clipping
    LD     A, 11111111b         ; Reset clipping mask
    LD     (clip_mask), A
    LD     A, E                 ; If ypos is negative
    OR     A                    ; try clipping the top
    JP     M, ClipTop           ;
 
    SUB    64                   ; If ypos is >= 64
    RET    NC                   ; sprite is off-screen

    NEG                         ; If (64 - ypos) > height
    CP     B                    ; don't need to clip
    JR     NC, VertClipDone     ; 

    LD     B, A                 ; Do bottom clipping by
    JR     VertClipDone         ; setting height to (64 - ypos)

ClipTopOR:
    LD     A, B                 ; If ypos <= -height
    NEG                         ; sprite is off-screen
    SUB    E                    ;
    RET    NC                   ;

    PUSH   AF
    ADD    A, B                 ; Get the number of clipped rows
    LD     E, 0                 ; Set ypos to 0 (top of screen)
    LD     B, E                 ; Advance image data pointer
    LD     C, A                 ;
    ADD    IX, BC               ;
    POP    AF
    NEG                         ; Get the number of visible rows
    LD     B, A                 ; and set as height

VertClipDoneOR:
; Now we're doing horizontal clipping
    LD     C, 0                 ; Reset correction factor
    LD     A, D

    CP     -7                   ; If 0 > xpos >= -7
    JR     NC, ClipLeftOR       ; clip the left side

    CP     96                   ; If xpos >= 96
    RET    NC                   ; sprite is off-screen

    CP     89                   ; If 0 <= xpos < 89
    JR     C, HorizClipDoneOR   ; don't need to clip

ClipRightOR:
    AND    7                    ; Determine the clipping mask
    LD     C, A
    LD     A, 11111111b
FindRightMaskOR:
    ADD    A, A
    DEC    C
    JR     NZ, FindRightMaskOR
    LD     (clip_mask), A
    LD     A, D
    JR     HorizClipDoneOR

ClipLeftOR:
    AND    7                    ; Determine the clipping mask
    LD     C, A
    LD     A, 11111111b
FindLeftMaskOR:
    ADD    A, A
    DEC    C
    JR     NZ, FindLeftMaskOR
    CPL
    LD     (clip_mask), A
    LD     A, D
    ADD    A, 96                ; Set xpos so sprite will "spill over"
    LD     C, 12                ; Set correction

HorizClipDoneOR:
; A = xpos
; E = ypos
; B = height
; IX = image address

; Now we can finally display the sprite.
    LD     H, 0
    LD     D, H
    LD     L, E
    ADD    HL, HL
    ADD    HL, DE
    ADD    HL, HL
    ADD    HL, HL

    LD     E, A
    SRL    E
    SRL    E
    SRL    E
    ADD    HL, DE

    LD     DE, LCDBuffer
    ADD    HL, DE

    LD     D, 0                 ; Correct graph buffer address
    LD     E, C                 ; if clipping the left side
    SBC    HL, DE               ;

    AND    7
    JR     Z, _AlignedOR

    LD     C, A
    LD     DE, 11

_RowLoopOR:
    PUSH   BC
    LD     B, C
    LD     A, (clip_mask)       ; Mask out the part of the sprite
    AND    (IX)                 ; to be horizontally clipped
    LD     C, 0

_ShiftLoopOR:
    SRL    A
    RR     C
    DJNZ   _ShiftLoopOR

    or    (HL)
    LD     (HL), A

    INC    HL
    LD     A, C
    or    (HL)
    LD     (HL), A

    ADD    HL, DE
    INC    IX
    POP    BC
    DJNZ   _RowLoopOR
    RET

_AlignedOR:
    LD     DE, 12

_PutLoopOR:
    LD     A, (IX)
    XOR    (HL)
    LD     (HL), A
    INC    IX
    ADD    HL, DE
    DJNZ   _PutLoopOR
    RET
