 DEFINE PAGE00, SPACE=ROM
 SEGMENT PAGE00

 public WaitForever, DEMulA, CmpStrings, Sleep, CPDEBC, Quicksort, Random
 extern LCDDelay, ClearBuffer, FastCopy

 include "WFRNGOS.inc"

CPDEBC:
	ld a, h
	cp b
	ret nz
	ld a, l
	cp c
	ret

Sleep:
	call LCDDelay ; Just to be safe
	ld a, 2
	out (10h), a ; Disable LCD
	di ; And interrupts, for now
	im 1 ; interrupt mode 1, for cleanliness
	ei ; Enable interrupting when ON is pressed
	ld a, 1h
	out (03h), a ; ON
	halt ; and halt :)
	ld a, 0Bh ; Reset the interrupts
	out (03h), a
	ld a, 3
	out (10h), a ; Enable the screen
	ret

WaitForever:
	jr WaitForever

DEMulA:          ; HL = DE  A
    LD     HL, 0      ; Use HL to store the product
    LD     B, 8       ; Eight bits to check
_loop:
    RRCA             ; Check least-significant bit of accumulator
    JR     NC, _skip  ; If zero, skip addition
    ADD    HL, DE
_skip:
    SLA    E         ; Shift DE one bit left
    RL     D
    DJNZ   _loop
    RET
; Input: HL and DE are strings
; Output: Z if equal, NZ otherwise
CmpStrings:
	ld a, (de)
	cp 0
	jr z, CmpStringsEoS
	cp (hl)
	ret nz
	inc hl
	inc de
	jr CmpStrings
CmpStringsEoS:
	ld a, (hl)
	cp 0
	ret

;
; >>> Quicksort routine v1.1 <<<
; by Frank Yaul 7/14/04
;
; Usage: bc->first, de->last,
;        call qsort
; Destroys: abcdefhl
;
Quicksort:
		push hl
		push de
		push bc
		ld      hl,0
        push    hl
qsloop: ld      h,b
        ld      l,c
        or      a
        sbc     hl,de
        jp      c,next1 ;loop until lo<hi
        pop     bc
        ld      a,b
        or      c
        jr z, endqsort
        pop     de
        jp      qsloop
next1:  push    de      ;save hi,lo
        push    bc
        ld      a,(bc)  ;pivot
        ld      h,a
        dec     bc
        inc     de
fleft:  inc     bc      ;do i++ while cur<piv
        ld      a,(bc)
        cp      h
        jp      c,fleft
fright: dec     de      ;do i-- while cur>piv
        ld      a,(de)
        ld      l,a
        ld      a,h
        cp      l
        jp      c,fright
        push    hl      ;save pivot
        ld      h,d     ;exit if lo>hi
        ld      l,e
        or      a
        sbc     hl,bc
        jp      c,next2
        ld      a,(bc)  ;swap (bc),(de)
        ld      h,a
        ld      a,(de)
        ld      (bc),a
        ld      a,h
        ld      (de),a
        pop     hl      ;restore pivot
        jp      fleft
next2:  pop     hl      ;restore pivot
        pop     hl      ;pop lo
        push    bc      ;stack=left-hi
        ld      b,h
        ld      c,l     ;bc=lo,de=right
        jp      qsloop
endqsort:
		pop bc
		pop de
		pop hl
		ret
;
; >>> end Quicksort <<<
;

Random:
; Uses a 16-bit RAM variable called RandomNumberGeneratorWord
; Returns an 8-bit pseudo-random number in a
	    push hl
        ld hl,(RandomSeed)
        ld a,h         ; get high byte
        rrca           ; rotate right by 2
        rrca
        xor h          ; xor with original
        rrca           ; rotate right by 1
        xor l          ; xor with low byte
        rrca           ; rotate right by 4
        rrca
        rrca
        rrca
        xor l          ; xor again
        rra            ; rotate right by 1 through carry
        adc hl,hl      ; add RandomNumberGeneratorWord to itself
        jr nz,next
        ld hl,0733Ch    ; if last xor resulted in zero then re-seed random number generator
next:   ld a,r         ; r = refresh register = semi-random number
        xor l          ; xor with l which is fairly random
        ld (RandomSeed),hl
    pop hl
    ret                ; return random number in a
